import { ListRange } from '@angular/cdk/collections';
import { FlatTreeControl } from '@angular/cdk/tree';
import { AfterViewInit, ChangeDetectorRef, ElementRef, OnChanges, OnDestroy, Renderer2, TrackByFunction } from '@angular/core';
import { Nullable } from '@bimeister/utilities';
import { BehaviorSubject, Observable } from 'rxjs';
import { FlatTreeItem } from '../../../../declarations/classes/flat-tree-item.class';
import { TreeRangedDataSource } from '../../../../declarations/classes/tree-ranged-data-source.class';
import { TreeController } from '../../../../declarations/classes/tree-controller.class';
import { TreeDataDisplayCollectionRef } from '../../../../declarations/interfaces/tree-data-display-collection-ref.interface';
import { TreeItemTemplateDirective } from '../../directives/tree-item-template.directive';
import { TreeNodeProperties } from '../../../../declarations/interfaces/tree-node-properties.interface';
import { CdkVirtualScrollViewport } from '@angular/cdk/scrolling';
import { ComponentChanges } from '@bimeister/pupakit.common';
import * as i0 from "@angular/core";
interface Position {
    top: number;
    left: number;
}
declare type ScrollDirection = null | 'up' | 'down';
interface DragAndDropMeta {
    dragTreeItem: FlatTreeItem;
    dropTreeItem?: FlatTreeItem;
    dragTreeItemLeft?: number;
    dragTreeItemTop?: number;
    dragTreeItemWidth?: number;
    mouseDownPosition?: Position;
    scrollDirection?: ScrollDirection;
    draggableElementBoundingBox?: DOMRect;
    dragTreeItemIsDisplayed: boolean;
}
export declare class TreeNewComponent<T> implements AfterViewInit, OnChanges, OnDestroy {
    readonly renderer: Renderer2;
    readonly host: ElementRef<HTMLElement>;
    private readonly changeDetectorRef;
    controller: TreeController;
    readonly skeletonViewPort: CdkVirtualScrollViewport;
    readonly viewPort: CdkVirtualScrollViewport;
    readonly treeItemTemplate: TreeItemTemplateDirective<T>;
    dataSource: TreeRangedDataSource;
    dataDisplayCollection: TreeDataDisplayCollectionRef;
    data$: Observable<FlatTreeItem[]>;
    expandedIdsList$: Observable<string[]>;
    selectedIdsList$: Observable<string[]>;
    isLoading$: Observable<boolean>;
    hasDragAndDrop$: Observable<boolean>;
    trackBy$: Observable<TrackByFunction<FlatTreeItem>>;
    treeItemSizePx$: Observable<number>;
    private scrollBehavior$;
    private eventBus;
    readonly treeControl: FlatTreeControl<FlatTreeItem>;
    readonly listRange$: BehaviorSubject<ListRange>;
    readonly dragAndDropMeta$: BehaviorSubject<Nullable<DragAndDropMeta>>;
    private readonly expandWithDelay$;
    private readonly subscription;
    constructor(renderer: Renderer2, host: ElementRef<HTMLElement>, changeDetectorRef: ChangeDetectorRef);
    ngAfterViewInit(): void;
    ngOnChanges(changes: ComponentChanges<this>): void;
    ngOnDestroy(): void;
    isExpanded(expandedIdsList: string[], treeItem: FlatTreeItem): boolean;
    hasChild(_: number, treeItem: FlatTreeItem): boolean;
    hasNoChild(_: number, treeItem: FlatTreeItem): boolean;
    isElement(_: number, treeItem: FlatTreeItem): boolean;
    idsIncludesNodeId(idsList: string[], treeItem: FlatTreeItem): boolean;
    mouseDown(dragTreeItem: FlatTreeItem, event: MouseEvent): void;
    mouseEnter(treeItem: FlatTreeItem): void;
    mouseLeave(): void;
    mouseMove(event: MouseEvent): void;
    mouseUp(): void;
    click(treeItem: FlatTreeItem): void;
    toggleExpansion(expandedIdsList: string[], treeItem: FlatTreeItem): void;
    getTreeNodeProperties(node: FlatTreeItem): Observable<TreeNodeProperties>;
    private setupController;
    private expandClick;
    private collapseClick;
    private getSubscriptionToExpandWhileDragging;
    private getSubscriptionToSetLoading;
    private getSubscriptionToSetExpanded;
    private setExpandedIdsList;
    private getSubscriptionToSetSelected;
    private setSelectedIdsList;
    private getSubscriptionToScrollToIndex;
    private scrollToIndex;
    private getSubscriptionForScrollWithDrag;
    private getSubscriptionForRangeChanges;
    private getEvents;
    private processDragMoving;
    private static getLevel;
    private static isExpandable;
    static ɵfac: i0.ɵɵFactoryDeclaration<TreeNewComponent<any>, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<TreeNewComponent<any>, "pupa-tree-new", never, { "controller": "controller"; }, {}, ["treeItemTemplate"], ["*"], false>;
}
export {};
