import * as i0 from '@angular/core';
import { Component, ViewEncapsulation, ChangeDetectionStrategy, Input, Directive, ContentChild, NgModule, ViewChild, HostListener, Injectable } from '@angular/core';
import * as i4 from '@angular/cdk/tree';
import { CdkTreeModule, FlatTreeControl } from '@angular/cdk/tree';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i1$1 from '@bimeister/pupakit.icons';
import { PupaIconsModule, appCircle, appChevronDownIcon, appChevronRightIcon, iosArrowDownIcon, iosArrowForwardIcon } from '@bimeister/pupakit.icons';
import * as i2 from '@bimeister/pupakit.kit';
import { PupaButtonsModule, PupaCheckboxModule, PupaSpinnerModule, PupaSkeletonModule, PupaScrollableModule } from '@bimeister/pupakit.kit';
import { BehaviorSubject, combineLatest, Subject, Subscription, NEVER, timer, interval, animationFrameScheduler } from 'rxjs';
import { isNil, shareReplayWithRefCount, getUuid, filterTruthy, filterNotNil, getClampedValue } from '@bimeister/utilities';
import * as i3 from '@angular/cdk/scrolling';
import { ScrollingModule } from '@angular/cdk/scrolling';
import { DataSource } from '@angular/cdk/collections';
import { EventBus } from '@bimeister/event-bus/rxjs';
import { filter, map, take, debounce, observeOn, switchMap, withLatestFrom, mapTo } from 'rxjs/operators';
import { BusEventBase, QueueEvents, EventsQueue } from '@bimeister/pupakit.common';

const TREE_NODE_OFFSET = 16;

class TreeNodeSkeletonComponent {
    get offset() {
        return `${this.level * TREE_NODE_OFFSET}px`;
    }
}
TreeNodeSkeletonComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNodeSkeletonComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
TreeNodeSkeletonComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.2.12", type: TreeNodeSkeletonComponent, selector: "pupa-tree-node-skeleton", inputs: { level: "level" }, ngImport: i0, template: "<div class=\"tree-node-skeleton\">\n  <div [style.margin-left]=\"offset\" class=\"tree-node-skeleton__left\"></div>\n  <div class=\"tree-node-skeleton__center\"></div>\n  <div class=\"tree-node-skeleton__right\"></div>\n</div>\n", styles: [":host{display:block;width:100%}.tree-node-skeleton{display:flex;padding:1rem 2rem}.tree-node-skeleton__left,.tree-node-skeleton__center,.tree-node-skeleton__right{background-color:rgba(var(--color_neutral-4),var(--alpha-1000))}.tree-node-skeleton__left{min-width:6rem;height:6rem;border-radius:.5rem}.tree-node-skeleton__center{width:100%;height:6rem;margin:0 3rem;border-radius:1rem}.tree-node-skeleton__right{min-width:6rem;height:6rem;border-radius:1rem}:host-context(.dark-theme) .tree-node-skeleton__left,:host-context(.dark-theme) .tree-node-skeleton__center,:host-context(.dark-theme) .tree-node-skeleton__right{background-color:rgba(var(--color_neutral-400),var(--alpha-1000))}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNodeSkeletonComponent, decorators: [{
            type: Component,
            args: [{ selector: 'pupa-tree-node-skeleton', encapsulation: ViewEncapsulation.Emulated, changeDetection: ChangeDetectionStrategy.OnPush, template: "<div class=\"tree-node-skeleton\">\n  <div [style.margin-left]=\"offset\" class=\"tree-node-skeleton__left\"></div>\n  <div class=\"tree-node-skeleton__center\"></div>\n  <div class=\"tree-node-skeleton__right\"></div>\n</div>\n", styles: [":host{display:block;width:100%}.tree-node-skeleton{display:flex;padding:1rem 2rem}.tree-node-skeleton__left,.tree-node-skeleton__center,.tree-node-skeleton__right{background-color:rgba(var(--color_neutral-4),var(--alpha-1000))}.tree-node-skeleton__left{min-width:6rem;height:6rem;border-radius:.5rem}.tree-node-skeleton__center{width:100%;height:6rem;margin:0 3rem;border-radius:1rem}.tree-node-skeleton__right{min-width:6rem;height:6rem;border-radius:1rem}:host-context(.dark-theme) .tree-node-skeleton__left,:host-context(.dark-theme) .tree-node-skeleton__center,:host-context(.dark-theme) .tree-node-skeleton__right{background-color:rgba(var(--color_neutral-400),var(--alpha-1000))}\n"] }]
        }], propDecorators: { level: [{
                type: Input
            }] } });

class TreeNodeActionsDirective {
    constructor(templateRef) {
        this.templateRef = templateRef;
        this.isDisplayed$ = new BehaviorSubject(true);
    }
    ngOnChanges() {
        this.isDisplayed$.next(Boolean(this.pupaTreeNodeActions));
    }
}
TreeNodeActionsDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNodeActionsDirective, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
TreeNodeActionsDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "14.2.12", type: TreeNodeActionsDirective, selector: "[pupaTreeNodeActions]", inputs: { pupaTreeNodeActions: "pupaTreeNodeActions" }, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNodeActionsDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[pupaTreeNodeActions]',
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }]; }, propDecorators: { pupaTreeNodeActions: [{
                type: Input
            }] } });

class TreeNodeComponent {
    get arrowIcon() {
        if (this.treeNodeProperties.isExpanded) {
            return 'app-chevron-down';
        }
        return 'app-chevron-right';
    }
    get arrowOffset() {
        return `${this.treeNodeProperties.level * TREE_NODE_OFFSET}px`;
    }
    get isSelected() {
        return this.treeNodeProperties.isSelected || this.treeNodeProperties.isHighlighted;
    }
    expand(event) {
        event.stopPropagation();
        if (isNil(this.treeNodeProperties.expand)) {
            return;
        }
        this.treeNodeProperties.expand();
    }
}
TreeNodeComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNodeComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
TreeNodeComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.2.12", type: TreeNodeComponent, selector: "pupa-tree-node", inputs: { treeNodeProperties: "treeNodeProperties" }, queries: [{ propertyName: "treeNodeActionsTemplate", first: true, predicate: TreeNodeActionsDirective, descendants: true }], ngImport: i0, template: "<div\n  class=\"tree-node\"\n  [class.tree-node--selected]=\"isSelected\"\n  [class.tree-node--disabled]=\"treeNodeProperties.isDisabled\"\n  [class.tree-node--expanded]=\"treeNodeProperties.isExpanded\"\n>\n  <svg\n    class=\"tree-node__selection\"\n    width=\"2\"\n    height=\"31\"\n    viewBox=\"0 0 2 31\"\n    fill=\"none\"\n    xmlns=\"http://www.w3.org/2000/svg\"\n  >\n    <path d=\"M2 31V0C0.4 0 0 2 0 3V28C0 30.4 1.33333 31 2 31Z\" fill=\"currentColor\" />\n  </svg>\n  <div class=\"tree-node__left\">\n    <div [style.margin-left]=\"arrowOffset\" class=\"tree-node__arrow-container\">\n      <ng-container *ngIf=\"treeNodeProperties.isDirectory\">\n        <pupa-button-icon\n          (click)=\"expand($event)\"\n          *ngIf=\"treeNodeProperties.hasChildren\"\n          [icon]=\"arrowIcon\"\n          [disabled]=\"treeNodeProperties.isDisabled\"\n          [loading]=\"treeNodeProperties.isLoading\"\n          size=\"s\"\n          kind=\"secondary\"\n        ></pupa-button-icon>\n        <pupa-icon\n          *ngIf=\"!treeNodeProperties.hasChildren\"\n          class=\"tree-node__without-children\"\n          name=\"app-circle\"\n        ></pupa-icon>\n      </ng-container>\n    </div>\n    <div class=\"tree-node__text\">\n      <ng-content></ng-content>\n    </div>\n  </div>\n  <div class=\"tree-node__right\">\n    <div\n      *ngIf=\"treeNodeActionsTemplate?.isDisplayed$ | async\"\n      (click)=\"$event.stopPropagation()\"\n      class=\"tree-node__right-container\"\n    >\n      <ng-container [ngTemplateOutlet]=\"treeNodeActionsTemplate?.templateRef\"></ng-container>\n    </div>\n  </div>\n</div>\n", styles: [":host{display:block;height:8rem}.tree-node{display:flex;flex-direction:row;align-items:center;justify-content:normal;flex-wrap:nowrap;min-width:48.5rem;height:100%;background-color:rgba(var(--color_neutral-1),var(--alpha-1000));border-radius:1rem;cursor:pointer;-webkit-appearance:none;-moz-appearance:none;appearance:none;-webkit-tap-highlight-color:transparent;pointer-events:auto;-webkit-user-select:none;user-select:none}.tree-node__selection{position:sticky;left:0;width:2px;min-width:2px;height:8rem;opacity:0}.tree-node__left{display:flex;flex-direction:row;align-items:center;justify-content:normal;flex-wrap:nowrap;overflow:hidden;padding:1rem 0 1rem 1.5rem;margin-right:-4rem}.tree-node__right{display:flex;flex-direction:row;align-items:normal;justify-content:normal;flex-wrap:nowrap;position:sticky;right:0;height:6rem;margin:1rem 0 1rem auto;padding-left:4rem;padding-right:2rem;background:linear-gradient(90deg,rgba(var(--color_neutral-1),var(--alpha-500)) -95%,rgba(var(--color_neutral-1),var(--alpha-1000)) 70%)}.tree-node__right-container{display:flex;flex-direction:row;align-items:center;justify-content:flex-end;flex-wrap:nowrap;position:relative;z-index:2;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content}.tree-node__arrow-container{display:flex;flex-direction:row;align-items:center;justify-content:center;flex-wrap:nowrap;min-width:9rem;height:6rem;padding-right:3rem;box-sizing:border-box}.tree-node__without-children{width:1rem;height:1rem;color:rgba(var(--color_neutral-60),var(--alpha-1000))}.tree-node__text{font-family:NotoSans,sans-serif;font-weight:400;font-size:11px;line-height:16px;color:rgba(var(--color_neutral-1000),var(--alpha-1000));white-space:nowrap;padding-right:4rem}@media (hover: hover){.tree-node:hover{background-color:rgba(var(--color_neutral-6),var(--alpha-1000))}.tree-node:hover .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-6),var(--alpha-500)) -95%,rgba(var(--color_neutral-6),var(--alpha-1000)) 70%)}}.tree-node:active{background-color:rgba(var(--color_neutral-7),var(--alpha-1000))}.tree-node:active .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-7),var(--alpha-500)) -95%,rgba(var(--color_neutral-7),var(--alpha-1000)) 70%)}.tree-node:focus{border:2px solid rgba(var(--color_primary-500),var(--alpha-1000))}.tree-node--dragover{border:1px solid rgba(var(--color_primary-500),var(--alpha-1000))}.tree-node--selected{background-color:rgba(var(--color_neutral-4),var(--alpha-1000))}.tree-node--selected .tree-node__selection{opacity:1;color:rgba(var(--color_primary-500),var(--alpha-1000))}.tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-4),var(--alpha-500)) -95%,rgba(var(--color_neutral-4),var(--alpha-1000)) 70%)}.tree-node--disabled.tree-node{cursor:not-allowed;background-color:rgba(var(--color_neutral-1),var(--alpha-1000))}.tree-node--disabled.tree-node .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-1),var(--alpha-500)) -95%,rgba(var(--color_neutral-1),var(--alpha-1000)) 70%)}.tree-node--disabled.tree-node .tree-node__text{color:rgba(var(--color_neutral-60),var(--alpha-1000))}.tree-node--disabled.tree-node--selected{background-color:rgba(var(--color_neutral-4),var(--alpha-1000))}.tree-node--disabled.tree-node--selected .tree-node__selection{color:rgba(var(--color_primary-500),var(--alpha-1000));opacity:1}.tree-node--disabled.tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-4),var(--alpha-500)) -95%,rgba(var(--color_neutral-4),var(--alpha-1000)) 70%)}.tree-node--expanded .tree-node__text{font-family:NotoSans,sans-serif;font-weight:500;font-size:11px;line-height:16px}:host-context(.dark-theme) .tree-node{background-color:rgba(var(--color_neutral-1000),var(--alpha-1000))}:host-context(.dark-theme) .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-1000),var(--alpha-500)) -95%,rgba(var(--color_neutral-1000),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node__text{color:rgba(var(--color_neutral-1),var(--alpha-1000))}@media (hover: hover){:host-context(.dark-theme) .tree-node:hover{background-color:rgba(var(--color_neutral-600),var(--alpha-1000))}:host-context(.dark-theme) .tree-node:hover .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-600),var(--alpha-500)) -95%,rgba(var(--color_neutral-600),var(--alpha-1000)) 70%)}}:host-context(.dark-theme) .tree-node:active{background-color:rgba(var(--color_neutral-700),var(--alpha-1000))}:host-context(.dark-theme) .tree-node:active .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-700),var(--alpha-500)) -95%,rgba(var(--color_neutral-700),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node--selected{background-color:rgba(var(--color_neutral-500),var(--alpha-1000))}:host-context(.dark-theme) .tree-node--selected .tree-node__selection{color:rgba(var(--color_primary-40),var(--alpha-1000))}:host-context(.dark-theme) .tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-500),var(--alpha-500)) -95%,rgba(var(--color_neutral-500),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node.tree-node--disabled{background-color:rgba(var(--color_neutral-1000),var(--alpha-1000))}:host-context(.dark-theme) .tree-node.tree-node--disabled .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-1000),var(--alpha-500)) -95%,rgba(var(--color_neutral-1000),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node.tree-node--disabled .tree-node__text{color:rgba(var(--color_neutral-60),var(--alpha-1000))}:host-context(.dark-theme) .tree-node.tree-node--disabled.tree-node--selected{background-color:rgba(var(--color_neutral-500),var(--alpha-1000))}:host-context(.dark-theme) .tree-node.tree-node--disabled.tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-500),var(--alpha-500)) -95%,rgba(var(--color_neutral-500),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node.tree-node--disabled.tree-node--selected .tree-node__text{color:rgba(var(--color_neutral-60),var(--alpha-1000))}\n"], dependencies: [{ kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "component", type: i2.ButtonIconComponent, selector: "pupa-button-icon", inputs: ["size", "kind", "type", "disabled", "icon", "loading", "active", "tabIndex"] }, { kind: "component", type: i1$1.IconComponent, selector: "pupa-icon", inputs: ["name"] }, { kind: "pipe", type: i1.AsyncPipe, name: "async" }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNodeComponent, decorators: [{
            type: Component,
            args: [{ selector: 'pupa-tree-node', encapsulation: ViewEncapsulation.Emulated, changeDetection: ChangeDetectionStrategy.OnPush, template: "<div\n  class=\"tree-node\"\n  [class.tree-node--selected]=\"isSelected\"\n  [class.tree-node--disabled]=\"treeNodeProperties.isDisabled\"\n  [class.tree-node--expanded]=\"treeNodeProperties.isExpanded\"\n>\n  <svg\n    class=\"tree-node__selection\"\n    width=\"2\"\n    height=\"31\"\n    viewBox=\"0 0 2 31\"\n    fill=\"none\"\n    xmlns=\"http://www.w3.org/2000/svg\"\n  >\n    <path d=\"M2 31V0C0.4 0 0 2 0 3V28C0 30.4 1.33333 31 2 31Z\" fill=\"currentColor\" />\n  </svg>\n  <div class=\"tree-node__left\">\n    <div [style.margin-left]=\"arrowOffset\" class=\"tree-node__arrow-container\">\n      <ng-container *ngIf=\"treeNodeProperties.isDirectory\">\n        <pupa-button-icon\n          (click)=\"expand($event)\"\n          *ngIf=\"treeNodeProperties.hasChildren\"\n          [icon]=\"arrowIcon\"\n          [disabled]=\"treeNodeProperties.isDisabled\"\n          [loading]=\"treeNodeProperties.isLoading\"\n          size=\"s\"\n          kind=\"secondary\"\n        ></pupa-button-icon>\n        <pupa-icon\n          *ngIf=\"!treeNodeProperties.hasChildren\"\n          class=\"tree-node__without-children\"\n          name=\"app-circle\"\n        ></pupa-icon>\n      </ng-container>\n    </div>\n    <div class=\"tree-node__text\">\n      <ng-content></ng-content>\n    </div>\n  </div>\n  <div class=\"tree-node__right\">\n    <div\n      *ngIf=\"treeNodeActionsTemplate?.isDisplayed$ | async\"\n      (click)=\"$event.stopPropagation()\"\n      class=\"tree-node__right-container\"\n    >\n      <ng-container [ngTemplateOutlet]=\"treeNodeActionsTemplate?.templateRef\"></ng-container>\n    </div>\n  </div>\n</div>\n", styles: [":host{display:block;height:8rem}.tree-node{display:flex;flex-direction:row;align-items:center;justify-content:normal;flex-wrap:nowrap;min-width:48.5rem;height:100%;background-color:rgba(var(--color_neutral-1),var(--alpha-1000));border-radius:1rem;cursor:pointer;-webkit-appearance:none;-moz-appearance:none;appearance:none;-webkit-tap-highlight-color:transparent;pointer-events:auto;-webkit-user-select:none;user-select:none}.tree-node__selection{position:sticky;left:0;width:2px;min-width:2px;height:8rem;opacity:0}.tree-node__left{display:flex;flex-direction:row;align-items:center;justify-content:normal;flex-wrap:nowrap;overflow:hidden;padding:1rem 0 1rem 1.5rem;margin-right:-4rem}.tree-node__right{display:flex;flex-direction:row;align-items:normal;justify-content:normal;flex-wrap:nowrap;position:sticky;right:0;height:6rem;margin:1rem 0 1rem auto;padding-left:4rem;padding-right:2rem;background:linear-gradient(90deg,rgba(var(--color_neutral-1),var(--alpha-500)) -95%,rgba(var(--color_neutral-1),var(--alpha-1000)) 70%)}.tree-node__right-container{display:flex;flex-direction:row;align-items:center;justify-content:flex-end;flex-wrap:nowrap;position:relative;z-index:2;width:-webkit-fit-content;width:-moz-fit-content;width:fit-content}.tree-node__arrow-container{display:flex;flex-direction:row;align-items:center;justify-content:center;flex-wrap:nowrap;min-width:9rem;height:6rem;padding-right:3rem;box-sizing:border-box}.tree-node__without-children{width:1rem;height:1rem;color:rgba(var(--color_neutral-60),var(--alpha-1000))}.tree-node__text{font-family:NotoSans,sans-serif;font-weight:400;font-size:11px;line-height:16px;color:rgba(var(--color_neutral-1000),var(--alpha-1000));white-space:nowrap;padding-right:4rem}@media (hover: hover){.tree-node:hover{background-color:rgba(var(--color_neutral-6),var(--alpha-1000))}.tree-node:hover .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-6),var(--alpha-500)) -95%,rgba(var(--color_neutral-6),var(--alpha-1000)) 70%)}}.tree-node:active{background-color:rgba(var(--color_neutral-7),var(--alpha-1000))}.tree-node:active .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-7),var(--alpha-500)) -95%,rgba(var(--color_neutral-7),var(--alpha-1000)) 70%)}.tree-node:focus{border:2px solid rgba(var(--color_primary-500),var(--alpha-1000))}.tree-node--dragover{border:1px solid rgba(var(--color_primary-500),var(--alpha-1000))}.tree-node--selected{background-color:rgba(var(--color_neutral-4),var(--alpha-1000))}.tree-node--selected .tree-node__selection{opacity:1;color:rgba(var(--color_primary-500),var(--alpha-1000))}.tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-4),var(--alpha-500)) -95%,rgba(var(--color_neutral-4),var(--alpha-1000)) 70%)}.tree-node--disabled.tree-node{cursor:not-allowed;background-color:rgba(var(--color_neutral-1),var(--alpha-1000))}.tree-node--disabled.tree-node .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-1),var(--alpha-500)) -95%,rgba(var(--color_neutral-1),var(--alpha-1000)) 70%)}.tree-node--disabled.tree-node .tree-node__text{color:rgba(var(--color_neutral-60),var(--alpha-1000))}.tree-node--disabled.tree-node--selected{background-color:rgba(var(--color_neutral-4),var(--alpha-1000))}.tree-node--disabled.tree-node--selected .tree-node__selection{color:rgba(var(--color_primary-500),var(--alpha-1000));opacity:1}.tree-node--disabled.tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-4),var(--alpha-500)) -95%,rgba(var(--color_neutral-4),var(--alpha-1000)) 70%)}.tree-node--expanded .tree-node__text{font-family:NotoSans,sans-serif;font-weight:500;font-size:11px;line-height:16px}:host-context(.dark-theme) .tree-node{background-color:rgba(var(--color_neutral-1000),var(--alpha-1000))}:host-context(.dark-theme) .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-1000),var(--alpha-500)) -95%,rgba(var(--color_neutral-1000),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node__text{color:rgba(var(--color_neutral-1),var(--alpha-1000))}@media (hover: hover){:host-context(.dark-theme) .tree-node:hover{background-color:rgba(var(--color_neutral-600),var(--alpha-1000))}:host-context(.dark-theme) .tree-node:hover .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-600),var(--alpha-500)) -95%,rgba(var(--color_neutral-600),var(--alpha-1000)) 70%)}}:host-context(.dark-theme) .tree-node:active{background-color:rgba(var(--color_neutral-700),var(--alpha-1000))}:host-context(.dark-theme) .tree-node:active .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-700),var(--alpha-500)) -95%,rgba(var(--color_neutral-700),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node--selected{background-color:rgba(var(--color_neutral-500),var(--alpha-1000))}:host-context(.dark-theme) .tree-node--selected .tree-node__selection{color:rgba(var(--color_primary-40),var(--alpha-1000))}:host-context(.dark-theme) .tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-500),var(--alpha-500)) -95%,rgba(var(--color_neutral-500),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node.tree-node--disabled{background-color:rgba(var(--color_neutral-1000),var(--alpha-1000))}:host-context(.dark-theme) .tree-node.tree-node--disabled .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-1000),var(--alpha-500)) -95%,rgba(var(--color_neutral-1000),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node.tree-node--disabled .tree-node__text{color:rgba(var(--color_neutral-60),var(--alpha-1000))}:host-context(.dark-theme) .tree-node.tree-node--disabled.tree-node--selected{background-color:rgba(var(--color_neutral-500),var(--alpha-1000))}:host-context(.dark-theme) .tree-node.tree-node--disabled.tree-node--selected .tree-node__right{background:linear-gradient(90deg,rgba(var(--color_neutral-500),var(--alpha-500)) -95%,rgba(var(--color_neutral-500),var(--alpha-1000)) 70%)}:host-context(.dark-theme) .tree-node.tree-node--disabled.tree-node--selected .tree-node__text{color:rgba(var(--color_neutral-60),var(--alpha-1000))}\n"] }]
        }], propDecorators: { treeNodeProperties: [{
                type: Input
            }], treeNodeActionsTemplate: [{
                type: ContentChild,
                args: [TreeNodeActionsDirective]
            }] } });

class PupaTreeLayoutModule {
}
PupaTreeLayoutModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeLayoutModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
PupaTreeLayoutModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeLayoutModule, declarations: [TreeNodeComponent, TreeNodeActionsDirective, TreeNodeSkeletonComponent], imports: [CdkTreeModule,
        CommonModule,
        PupaButtonsModule, i1$1.PupaIconsModule, PupaCheckboxModule,
        PupaSpinnerModule], exports: [TreeNodeActionsDirective, TreeNodeComponent, TreeNodeSkeletonComponent] });
PupaTreeLayoutModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeLayoutModule, imports: [CdkTreeModule,
        CommonModule,
        PupaButtonsModule,
        PupaIconsModule.forFeature([appCircle, appChevronDownIcon, appChevronRightIcon]),
        PupaCheckboxModule,
        PupaSpinnerModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeLayoutModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [TreeNodeComponent, TreeNodeActionsDirective, TreeNodeSkeletonComponent],
                    imports: [
                        CdkTreeModule,
                        CommonModule,
                        PupaButtonsModule,
                        PupaIconsModule.forFeature([appCircle, appChevronDownIcon, appChevronRightIcon]),
                        PupaCheckboxModule,
                        PupaSpinnerModule,
                    ],
                    exports: [TreeNodeActionsDirective, TreeNodeComponent, TreeNodeSkeletonComponent],
                }]
        }] });

class FlatTreeItem {
    constructor(isExpandable, name, level, id, originalData = null, isElement = false) {
        this.isExpandable = isExpandable;
        this.name = name;
        this.level = level;
        this.id = id;
        this.originalData = originalData;
        this.isElement = isElement;
    }
}

class TreeRangedDataSource extends DataSource {
    constructor(data$) {
        super();
        this.data$ = data$;
        this.range$ = new BehaviorSubject(null);
    }
    setRange(range) {
        this.range$.next(range);
    }
    connect() {
        return combineLatest([this.data$, this.range$]).pipe(filter(([data, range]) => Array.isArray(data) && !isNil(range)), map(([data, range]) => {
            const { start, end } = range;
            return data.slice(start, end);
        }), shareReplayWithRefCount());
    }
    disconnect() {
        return;
    }
}

var TreeEvents;
(function (TreeEvents) {
    class TreeEventBase extends BusEventBase {
        constructor(payload = null, fromId = null) {
            super();
            this.payload = payload;
            this.fromId = fromId;
            this.scope = null;
            this.id = getUuid();
            this.type = null;
        }
    }
    TreeEvents.TreeEventBase = TreeEventBase;
    class Click extends TreeEventBase {
    }
    TreeEvents.Click = Click;
    class RemoveItem extends TreeEventBase {
    }
    TreeEvents.RemoveItem = RemoveItem;
    class UpdateItem extends TreeEventBase {
    }
    TreeEvents.UpdateItem = UpdateItem;
    class ScrollById extends TreeEventBase {
    }
    TreeEvents.ScrollById = ScrollById;
    class Drop extends TreeEventBase {
    }
    TreeEvents.Drop = Drop;
    class ScrollByIndex extends TreeEventBase {
    }
    TreeEvents.ScrollByIndex = ScrollByIndex;
    class ScrollViewport extends TreeEventBase {
    }
    TreeEvents.ScrollViewport = ScrollViewport;
    class SetData extends TreeEventBase {
    }
    TreeEvents.SetData = SetData;
    class SetLoading extends TreeEventBase {
    }
    TreeEvents.SetLoading = SetLoading;
    class SetSelected extends TreeEventBase {
    }
    TreeEvents.SetSelected = SetSelected;
    class Collapse extends TreeEventBase {
    }
    TreeEvents.Collapse = Collapse;
    class Expand extends TreeEventBase {
    }
    TreeEvents.Expand = Expand;
    class ExpandWhileDragging extends TreeEventBase {
    }
    TreeEvents.ExpandWhileDragging = ExpandWhileDragging;
    class RemoveChildren extends TreeEventBase {
    }
    TreeEvents.RemoveChildren = RemoveChildren;
    class SetExpanded extends TreeEventBase {
    }
    TreeEvents.SetExpanded = SetExpanded;
    class SetChildren extends TreeEventBase {
    }
    TreeEvents.SetChildren = SetChildren;
})(TreeEvents || (TreeEvents = {}));

class TreeItemTemplateDirective {
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
}
TreeItemTemplateDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeItemTemplateDirective, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
TreeItemTemplateDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "14.2.12", type: TreeItemTemplateDirective, selector: "[pupaTreeItemTemplate]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeItemTemplateDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[pupaTreeItemTemplate]',
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }]; } });

const EXPAND_WHILE_DRAGGING_DELAY = 1000;
class TreeNewComponent {
    constructor(renderer, host, changeDetectorRef) {
        this.renderer = renderer;
        this.host = host;
        this.changeDetectorRef = changeDetectorRef;
        this.treeControl = new FlatTreeControl(TreeNewComponent.getLevel, TreeNewComponent.isExpandable);
        this.listRange$ = new BehaviorSubject(null);
        this.dragAndDropMeta$ = new BehaviorSubject(null);
        this.expandWithDelay$ = new Subject();
        this.subscription = new Subscription();
    }
    ngAfterViewInit() {
        this.subscription.add(this.getSubscriptionToSetLoading());
        this.subscription.add(this.getSubscriptionToScrollToIndex());
        this.subscription.add(this.getSubscriptionToSetExpanded());
        this.subscription.add(this.getSubscriptionToSetSelected());
        this.subscription.add(this.getSubscriptionForScrollWithDrag());
        this.subscription.add(this.getSubscriptionForRangeChanges());
        this.subscription.add(this.getSubscriptionToExpandWhileDragging());
        this.eventBus.dispatch(new QueueEvents.StartQueue());
    }
    ngOnChanges(changes) {
        this.setupController(changes === null || changes === void 0 ? void 0 : changes.controller);
    }
    ngOnDestroy() {
        this.subscription.unsubscribe();
    }
    isExpanded(expandedIdsList, treeItem) {
        return !isNil(treeItem) && treeItem.isExpandable && !treeItem.isElement && expandedIdsList.includes(treeItem.id);
    }
    hasChild(_, treeItem) {
        return !isNil(treeItem) && treeItem.isExpandable && !treeItem.isElement;
    }
    hasNoChild(_, treeItem) {
        return !isNil(treeItem) && !treeItem.isExpandable && !treeItem.isElement;
    }
    isElement(_, treeItem) {
        return !isNil(treeItem) && !treeItem.isExpandable && treeItem.isElement;
    }
    idsIncludesNodeId(idsList, treeItem) {
        return idsList.includes(treeItem.id);
    }
    mouseDown(dragTreeItem, event) {
        this.hasDragAndDrop$.pipe(take(1), filterTruthy()).subscribe(() => {
            const mouseDownPosition = { left: event === null || event === void 0 ? void 0 : event.screenX, top: event === null || event === void 0 ? void 0 : event.screenY };
            const target = event.target;
            const draggableElementBoundingBox = target instanceof HTMLElement ? target.getBoundingClientRect() : null;
            const dragTreeItemWidth = draggableElementBoundingBox === null || draggableElementBoundingBox === void 0 ? void 0 : draggableElementBoundingBox.width;
            this.dragAndDropMeta$.next({
                mouseDownPosition,
                dragTreeItem,
                dropTreeItem: dragTreeItem,
                draggableElementBoundingBox,
                dragTreeItemWidth,
                dragTreeItemIsDisplayed: false,
            });
        });
    }
    mouseEnter(treeItem) {
        this.dragAndDropMeta$.pipe(take(1), filterNotNil()).subscribe((dragAndDropMeta) => {
            this.dragAndDropMeta$.next(Object.assign(Object.assign({}, dragAndDropMeta), { dropTreeItem: treeItem }));
            this.expandWithDelay$.next(treeItem);
        });
    }
    mouseLeave() {
        this.dragAndDropMeta$.pipe(take(1), filterNotNil()).subscribe((dragAndDropMeta) => {
            this.dragAndDropMeta$.next(Object.assign(Object.assign({}, dragAndDropMeta), { dropTreeItem: null }));
        });
    }
    mouseMove(event) {
        this.dragAndDropMeta$.pipe(take(1), filterNotNil()).subscribe((dragAndDropMeta) => {
            this.processDragMoving(dragAndDropMeta, event.screenX, event.screenY);
        });
    }
    mouseUp() {
        this.dragAndDropMeta$.pipe(take(1), filterNotNil()).subscribe((dragAndDropMeta) => {
            if (!dragAndDropMeta.dragTreeItemIsDisplayed) {
                return;
            }
            const dropEventData = {
                draggedElement: dragAndDropMeta.dragTreeItem,
                droppedElement: dragAndDropMeta.dropTreeItem,
            };
            this.eventBus.dispatch(new TreeEvents.Drop(dropEventData));
        });
        this.expandWithDelay$.next(null);
        this.dragAndDropMeta$.next(null);
    }
    click(treeItem) {
        this.eventBus.dispatch(new TreeEvents.Click(treeItem));
    }
    toggleExpansion(expandedIdsList, treeItem) {
        this.isExpanded(expandedIdsList, treeItem) ? this.collapseClick(treeItem) : this.expandClick(treeItem);
    }
    getTreeNodeProperties(node) {
        return combineLatest([this.expandedIdsList$, this.selectedIdsList$]).pipe(map(([expandedIdsList, selectedIdsList]) => ({
            level: node.level,
            isDirectory: !node.isElement,
            hasChildren: node.isExpandable,
            isLoading: false,
            isExpanded: this.isExpanded(expandedIdsList, node),
            isSelected: this.idsIncludesNodeId(selectedIdsList, node),
            isHighlighted: this.idsIncludesNodeId(selectedIdsList, node),
            isDisabled: false,
            expand: () => {
                this.toggleExpansion(expandedIdsList, node);
            },
        })));
    }
    setupController(change) {
        const value = change === null || change === void 0 ? void 0 : change.currentValue;
        if (isNil(value)) {
            return;
        }
        this.controller = value;
        this.dataDisplayCollection = this.controller.getDataDisplayCollectionRef();
        this.eventBus = this.controller.eventBus;
        this.scrollBehavior$ = this.dataDisplayCollection.scrollBehavior$;
        this.trackBy$ = this.dataDisplayCollection.trackBy$;
        this.treeItemSizePx$ = this.dataDisplayCollection.treeItemSizePx$;
        this.data$ = this.dataDisplayCollection.data$;
        this.isLoading$ = this.dataDisplayCollection.isLoading$;
        this.selectedIdsList$ = this.dataDisplayCollection.selectedIdsList$;
        this.expandedIdsList$ = this.dataDisplayCollection.expandedIdsList$;
        this.hasDragAndDrop$ = this.dataDisplayCollection.hasDragAndDrop$;
        this.dataSource = new TreeRangedDataSource(this.data$);
    }
    expandClick(treeItem) {
        this.controller.expand(treeItem.id);
    }
    collapseClick(treeItem) {
        this.eventBus.dispatch(new TreeEvents.Collapse(treeItem.id));
    }
    getSubscriptionToExpandWhileDragging() {
        return this.expandWithDelay$
            .pipe(debounce((node) => (isNil(node) ? NEVER : timer(EXPAND_WHILE_DRAGGING_DELAY))), filter((treeItem) => treeItem.isExpandable && !treeItem.isElement))
            .subscribe((treeItem) => {
            this.eventBus.dispatch(new TreeEvents.ExpandWhileDragging(treeItem.id));
        });
    }
    getSubscriptionToSetLoading() {
        return this.getEvents(TreeEvents.SetLoading).subscribe((event) => {
            this.dataDisplayCollection.setIsLoading(event.payload);
            this.changeDetectorRef.detectChanges();
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    getSubscriptionToSetExpanded() {
        return this.getEvents(TreeEvents.SetExpanded).subscribe((event) => {
            this.setExpandedIdsList(event.payload);
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    setExpandedIdsList(list) {
        this.dataDisplayCollection.setExpandedIdsList(list);
        this.data$
            .pipe(take(1), map((data) => data.filter((treeItem) => list.includes(treeItem.id))))
            .subscribe((data) => {
            data.forEach((treeItem) => this.treeControl.expand(treeItem));
        });
        this.changeDetectorRef.markForCheck();
    }
    getSubscriptionToSetSelected() {
        return this.getEvents(TreeEvents.SetSelected).subscribe((event) => {
            this.setSelectedIdsList(event.payload);
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    setSelectedIdsList(list) {
        this.dataDisplayCollection.setSelectedIdsList(list);
        this.changeDetectorRef.detectChanges();
    }
    getSubscriptionToScrollToIndex() {
        return this.getEvents(TreeEvents.ScrollByIndex).subscribe((event) => {
            this.scrollToIndex(event.payload);
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    scrollToIndex(index) {
        this.scrollBehavior$.pipe(take(1)).subscribe((scrollBehavior) => {
            this.viewPort.scrollToIndex(index, scrollBehavior);
            this.skeletonViewPort.scrollToIndex(index, scrollBehavior);
        });
    }
    getSubscriptionForScrollWithDrag() {
        return interval(0)
            .pipe(observeOn(animationFrameScheduler), switchMap(() => this.dragAndDropMeta$.pipe(take(1))), filterNotNil(), map((dragAndDropMeta) => dragAndDropMeta.scrollDirection), filterNotNil(), withLatestFrom(this.scrollBehavior$))
            .subscribe(([scrollDirection, scrollBehavior]) => {
            const scrollingSpeed = 5;
            const offsetDelta = scrollDirection === 'up' ? -scrollingSpeed : scrollingSpeed;
            const currentOffset = this.viewPort.measureScrollOffset();
            this.viewPort.scrollToOffset(currentOffset + offsetDelta, scrollBehavior);
        });
    }
    getSubscriptionForRangeChanges() {
        return this.viewPort.renderedRangeStream.subscribe((range) => {
            this.dataSource.setRange(range);
        });
    }
    getEvents(eventType) {
        return this.eventBus
            .listen()
            .pipe(filter((event) => event instanceof eventType));
    }
    processDragMoving(dragAndDropMeta, x, y) {
        const draggableElementBoundingBox = dragAndDropMeta.draggableElementBoundingBox;
        const mouseDownPosition = dragAndDropMeta.mouseDownPosition;
        const draggableElementPositionShift = {
            left: (mouseDownPosition === null || mouseDownPosition === void 0 ? void 0 : mouseDownPosition.left) - (draggableElementBoundingBox === null || draggableElementBoundingBox === void 0 ? void 0 : draggableElementBoundingBox.left),
            top: (mouseDownPosition === null || mouseDownPosition === void 0 ? void 0 : mouseDownPosition.top) - (draggableElementBoundingBox === null || draggableElementBoundingBox === void 0 ? void 0 : draggableElementBoundingBox.top),
        };
        const bottomBorderPositionY = this.host.nativeElement.clientHeight - (draggableElementBoundingBox === null || draggableElementBoundingBox === void 0 ? void 0 : draggableElementBoundingBox.height);
        const newMeta = Object.assign(Object.assign({}, dragAndDropMeta), { dragTreeItemLeft: x - (draggableElementBoundingBox === null || draggableElementBoundingBox === void 0 ? void 0 : draggableElementBoundingBox.left) - draggableElementPositionShift.left, dragTreeItemTop: getClampedValue(y - draggableElementPositionShift.top, 0, bottomBorderPositionY) });
        const isTopBorderReached = newMeta.dragTreeItemTop <= (draggableElementBoundingBox === null || draggableElementBoundingBox === void 0 ? void 0 : draggableElementBoundingBox.height);
        const isBottomBorderReached = newMeta.dragTreeItemTop >= bottomBorderPositionY;
        newMeta.scrollDirection = isTopBorderReached ? 'up' : isBottomBorderReached ? 'down' : null;
        newMeta.dragTreeItemIsDisplayed = true;
        this.dragAndDropMeta$.next(newMeta);
    }
    static getLevel(treeItem) {
        return treeItem.level;
    }
    static isExpandable(treeItem) {
        return treeItem.isExpandable;
    }
}
TreeNewComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNewComponent, deps: [{ token: i0.Renderer2 }, { token: i0.ElementRef }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component });
TreeNewComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.2.12", type: TreeNewComponent, selector: "pupa-tree-new", inputs: { controller: "controller" }, host: { listeners: { "window:mousemove": "mouseMove($event)", "window:mouseup": "mouseUp()" } }, queries: [{ propertyName: "treeItemTemplate", first: true, predicate: TreeItemTemplateDirective, descendants: true }], viewQueries: [{ propertyName: "skeletonViewPort", first: true, predicate: ["skeletonViewPort"], descendants: true, static: true }, { propertyName: "viewPort", first: true, predicate: ["viewPort"], descendants: true, static: true }], usesOnChanges: true, ngImport: i0, template: "<section class=\"tree\">\n  <div class=\"tree__loader\" *ngIf=\"isLoading$ | async\">\n    <pupa-spinner size=\"40px\"></pupa-spinner>\n  </div>\n\n  <pupa-scrollable [invisibleScrollbars]=\"['vertical', 'horizontal']\" class=\"skeleton-scrollable\">\n    <cdk-virtual-scroll-viewport\n      pupaScrollableContent\n      #skeletonViewPort\n      [itemSize]=\"treeItemSizePx$ | async\"\n      class=\"viewport viewport_skeleton\"\n    >\n      <pupa-skeleton>\n        <pupa-skeleton-group *cdkVirtualFor=\"let item of data$ | async\">\n          <pupa-tree-node-skeleton [level]=\"item.level\"></pupa-tree-node-skeleton>\n        </pupa-skeleton-group>\n      </pupa-skeleton>\n    </cdk-virtual-scroll-viewport>\n  </pupa-scrollable>\n\n  <pupa-scrollable>\n    <cdk-virtual-scroll-viewport pupaScrollableContent #viewPort [itemSize]=\"treeItemSizePx$ | async\" class=\"viewport\">\n      <ng-container *cdkVirtualFor=\"let treeItem of data$ | async\"> </ng-container>\n      <cdk-tree\n        [dataSource]=\"dataSource\"\n        [treeControl]=\"treeControl\"\n        [trackBy]=\"trackBy$ | async\"\n        (mouseleave)=\"mouseLeave()\"\n        class=\"scrollable-box__container\"\n      >\n        <cdk-tree-node\n          *cdkTreeNodeDef=\"let node; when: isElement\"\n          (mousedown)=\"mouseDown(node, $event)\"\n          (mouseenter)=\"mouseEnter(node)\"\n        >\n          <ng-container\n            [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n            [ngTemplateOutletContext]=\"{\n              $implicit: node\n            }\"\n          ></ng-container>\n        </cdk-tree-node>\n\n        <cdk-tree-node\n          *cdkTreeNodeDef=\"let node; when: hasNoChild\"\n          (mousedown)=\"mouseDown(node, $event)\"\n          (mouseenter)=\"mouseEnter(node)\"\n        >\n          <ng-container\n            [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n            [ngTemplateOutletContext]=\"{\n              $implicit: node\n            }\"\n          ></ng-container>\n        </cdk-tree-node>\n\n        <cdk-tree-node\n          *cdkTreeNodeDef=\"let node; when: hasChild\"\n          (mousedown)=\"mouseDown(node, $event)\"\n          (mouseenter)=\"mouseEnter(node)\"\n        >\n          <ng-container\n            [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n            [ngTemplateOutletContext]=\"{\n              $implicit: node\n            }\"\n          ></ng-container>\n        </cdk-tree-node>\n      </cdk-tree>\n    </cdk-virtual-scroll-viewport>\n  </pupa-scrollable>\n\n  <ng-container *ngIf=\"dragAndDropMeta$ | async as dragAndDropMeta\">\n    <ng-container\n      [ngTemplateOutlet]=\"dragTreeItemTemplate\"\n      [ngTemplateOutletContext]=\"{\n        treeItem: dragAndDropMeta.dragTreeItem,\n        top: dragAndDropMeta.dragTreeItemTop,\n        left: dragAndDropMeta.dragTreeItemLeft,\n        width: dragAndDropMeta.dragTreeItemWidth,\n        isVisible: dragAndDropMeta.dragTreeItemIsDisplayed\n      }\"\n    ></ng-container>\n  </ng-container>\n</section>\n\n<ng-template\n  let-left=\"left\"\n  let-width=\"width\"\n  let-top=\"top\"\n  let-treeItem=\"treeItem\"\n  let-isVisible=\"isVisible\"\n  #dragTreeItemTemplate\n>\n  <div *ngIf=\"isVisible\" class=\"draggable\" [style.left.px]=\"left\" [style.top.px]=\"top\" [style.width.px]=\"width\">\n    <ng-container\n      [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n      [ngTemplateOutletContext]=\"{\n        $implicit: treeItem\n      }\"\n    ></ng-container>\n  </div>\n</ng-template>\n\n<ng-template #defaultTemplate let-node>\n  <pupa-tree-node (click)=\"click(node)\" [treeNodeProperties]=\"getTreeNodeProperties(node) | async\">{{\n    node?.name\n  }}</pupa-tree-node>\n</ng-template>\n\n<ng-content></ng-content>\n", styles: [":host{width:100%;height:100%;box-sizing:border-box;display:block;--pupa-scrollable-content_padding: 2rem 3rem 3rem 2rem}.tree{position:relative;height:100%;width:auto;-webkit-user-select:none;user-select:none}.tree__loader{width:100%;height:100%;background-color:rgba(var(--color_neutral-1),var(--alpha-500));position:absolute;top:0;left:0;z-index:10;display:flex;justify-content:center;align-items:center;color:rgba(var(--color_neutral-500),var(--alpha-1000))}.viewport{height:100%;background-color:rgba(var(--color_neutral-1),var(--alpha-1000))}.viewport_skeleton{width:100%;pointer-events:none}:host-context(.dark-theme) .viewport{background-color:rgba(var(--color_neutral-1000),var(--alpha-1000))}.skeleton-scrollable{top:0;left:0;position:absolute}.draggable{position:absolute;pointer-events:none;box-shadow:var(--box-shadow_hover);z-index:1}\n"], dependencies: [{ kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i1.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "component", type: i2.SkeletonComponent, selector: "pupa-skeleton", inputs: ["isActive"] }, { kind: "component", type: i2.SkeletonGroupComponent, selector: "pupa-skeleton-group" }, { kind: "component", type: i2.SpinnerComponent, selector: "pupa-spinner", inputs: ["size"] }, { kind: "directive", type: i3.CdkFixedSizeVirtualScroll, selector: "cdk-virtual-scroll-viewport[itemSize]", inputs: ["itemSize", "minBufferPx", "maxBufferPx"] }, { kind: "directive", type: i3.CdkVirtualForOf, selector: "[cdkVirtualFor][cdkVirtualForOf]", inputs: ["cdkVirtualForOf", "cdkVirtualForTrackBy", "cdkVirtualForTemplate", "cdkVirtualForTemplateCacheSize"] }, { kind: "component", type: i3.CdkVirtualScrollViewport, selector: "cdk-virtual-scroll-viewport", inputs: ["orientation", "appendOnly"], outputs: ["scrolledIndexChange"] }, { kind: "directive", type: i4.CdkTreeNodeDef, selector: "[cdkTreeNodeDef]", inputs: ["cdkTreeNodeDefWhen"] }, { kind: "component", type: i4.CdkTree, selector: "cdk-tree", inputs: ["dataSource", "treeControl", "trackBy"], exportAs: ["cdkTree"] }, { kind: "directive", type: i4.CdkTreeNode, selector: "cdk-tree-node", inputs: ["role"], exportAs: ["cdkTreeNode"] }, { kind: "component", type: TreeNodeComponent, selector: "pupa-tree-node", inputs: ["treeNodeProperties"] }, { kind: "component", type: TreeNodeSkeletonComponent, selector: "pupa-tree-node-skeleton", inputs: ["level"] }, { kind: "component", type: i2.ScrollableComponent, selector: "pupa-scrollable", inputs: ["invisibleScrollbars", "size", "position", "syncWith", "scrollDragMode", "scrollVisibilityMode"], outputs: ["scrollTopChanged", "scrollLeftChanged", "verticalScrollVisibilityChanged", "horizontalScrollVisibilityChanged", "scrolledToHorizontalStart", "scrolledToHorizontalEnd", "scrolledToVerticalStart", "scrolledToVerticalEnd", "contentDragStart", "contentDragEnd", "verticalScrollDirectionChanged"] }, { kind: "directive", type: i2.ScrollableContentDirective, selector: "[pupaScrollableContent]" }, { kind: "pipe", type: i1.AsyncPipe, name: "async" }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeNewComponent, decorators: [{
            type: Component,
            args: [{ selector: 'pupa-tree-new', changeDetection: ChangeDetectionStrategy.OnPush, template: "<section class=\"tree\">\n  <div class=\"tree__loader\" *ngIf=\"isLoading$ | async\">\n    <pupa-spinner size=\"40px\"></pupa-spinner>\n  </div>\n\n  <pupa-scrollable [invisibleScrollbars]=\"['vertical', 'horizontal']\" class=\"skeleton-scrollable\">\n    <cdk-virtual-scroll-viewport\n      pupaScrollableContent\n      #skeletonViewPort\n      [itemSize]=\"treeItemSizePx$ | async\"\n      class=\"viewport viewport_skeleton\"\n    >\n      <pupa-skeleton>\n        <pupa-skeleton-group *cdkVirtualFor=\"let item of data$ | async\">\n          <pupa-tree-node-skeleton [level]=\"item.level\"></pupa-tree-node-skeleton>\n        </pupa-skeleton-group>\n      </pupa-skeleton>\n    </cdk-virtual-scroll-viewport>\n  </pupa-scrollable>\n\n  <pupa-scrollable>\n    <cdk-virtual-scroll-viewport pupaScrollableContent #viewPort [itemSize]=\"treeItemSizePx$ | async\" class=\"viewport\">\n      <ng-container *cdkVirtualFor=\"let treeItem of data$ | async\"> </ng-container>\n      <cdk-tree\n        [dataSource]=\"dataSource\"\n        [treeControl]=\"treeControl\"\n        [trackBy]=\"trackBy$ | async\"\n        (mouseleave)=\"mouseLeave()\"\n        class=\"scrollable-box__container\"\n      >\n        <cdk-tree-node\n          *cdkTreeNodeDef=\"let node; when: isElement\"\n          (mousedown)=\"mouseDown(node, $event)\"\n          (mouseenter)=\"mouseEnter(node)\"\n        >\n          <ng-container\n            [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n            [ngTemplateOutletContext]=\"{\n              $implicit: node\n            }\"\n          ></ng-container>\n        </cdk-tree-node>\n\n        <cdk-tree-node\n          *cdkTreeNodeDef=\"let node; when: hasNoChild\"\n          (mousedown)=\"mouseDown(node, $event)\"\n          (mouseenter)=\"mouseEnter(node)\"\n        >\n          <ng-container\n            [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n            [ngTemplateOutletContext]=\"{\n              $implicit: node\n            }\"\n          ></ng-container>\n        </cdk-tree-node>\n\n        <cdk-tree-node\n          *cdkTreeNodeDef=\"let node; when: hasChild\"\n          (mousedown)=\"mouseDown(node, $event)\"\n          (mouseenter)=\"mouseEnter(node)\"\n        >\n          <ng-container\n            [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n            [ngTemplateOutletContext]=\"{\n              $implicit: node\n            }\"\n          ></ng-container>\n        </cdk-tree-node>\n      </cdk-tree>\n    </cdk-virtual-scroll-viewport>\n  </pupa-scrollable>\n\n  <ng-container *ngIf=\"dragAndDropMeta$ | async as dragAndDropMeta\">\n    <ng-container\n      [ngTemplateOutlet]=\"dragTreeItemTemplate\"\n      [ngTemplateOutletContext]=\"{\n        treeItem: dragAndDropMeta.dragTreeItem,\n        top: dragAndDropMeta.dragTreeItemTop,\n        left: dragAndDropMeta.dragTreeItemLeft,\n        width: dragAndDropMeta.dragTreeItemWidth,\n        isVisible: dragAndDropMeta.dragTreeItemIsDisplayed\n      }\"\n    ></ng-container>\n  </ng-container>\n</section>\n\n<ng-template\n  let-left=\"left\"\n  let-width=\"width\"\n  let-top=\"top\"\n  let-treeItem=\"treeItem\"\n  let-isVisible=\"isVisible\"\n  #dragTreeItemTemplate\n>\n  <div *ngIf=\"isVisible\" class=\"draggable\" [style.left.px]=\"left\" [style.top.px]=\"top\" [style.width.px]=\"width\">\n    <ng-container\n      [ngTemplateOutlet]=\"treeItemTemplate?.templateRef || defaultTemplate\"\n      [ngTemplateOutletContext]=\"{\n        $implicit: treeItem\n      }\"\n    ></ng-container>\n  </div>\n</ng-template>\n\n<ng-template #defaultTemplate let-node>\n  <pupa-tree-node (click)=\"click(node)\" [treeNodeProperties]=\"getTreeNodeProperties(node) | async\">{{\n    node?.name\n  }}</pupa-tree-node>\n</ng-template>\n\n<ng-content></ng-content>\n", styles: [":host{width:100%;height:100%;box-sizing:border-box;display:block;--pupa-scrollable-content_padding: 2rem 3rem 3rem 2rem}.tree{position:relative;height:100%;width:auto;-webkit-user-select:none;user-select:none}.tree__loader{width:100%;height:100%;background-color:rgba(var(--color_neutral-1),var(--alpha-500));position:absolute;top:0;left:0;z-index:10;display:flex;justify-content:center;align-items:center;color:rgba(var(--color_neutral-500),var(--alpha-1000))}.viewport{height:100%;background-color:rgba(var(--color_neutral-1),var(--alpha-1000))}.viewport_skeleton{width:100%;pointer-events:none}:host-context(.dark-theme) .viewport{background-color:rgba(var(--color_neutral-1000),var(--alpha-1000))}.skeleton-scrollable{top:0;left:0;position:absolute}.draggable{position:absolute;pointer-events:none;box-shadow:var(--box-shadow_hover);z-index:1}\n"] }]
        }], ctorParameters: function () { return [{ type: i0.Renderer2 }, { type: i0.ElementRef }, { type: i0.ChangeDetectorRef }]; }, propDecorators: { controller: [{
                type: Input
            }], skeletonViewPort: [{
                type: ViewChild,
                args: ['skeletonViewPort', { static: true }]
            }], viewPort: [{
                type: ViewChild,
                args: ['viewPort', { static: true }]
            }], treeItemTemplate: [{
                type: ContentChild,
                args: [TreeItemTemplateDirective]
            }], mouseMove: [{
                type: HostListener,
                args: ['window:mousemove', ['$event']]
            }], mouseUp: [{
                type: HostListener,
                args: ['window:mouseup']
            }] } });

class PupaTreeNewModule {
}
PupaTreeNewModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeNewModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
PupaTreeNewModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeNewModule, declarations: [TreeNewComponent, TreeItemTemplateDirective], imports: [CommonModule,
        PupaSkeletonModule,
        PupaSpinnerModule,
        ScrollingModule,
        CdkTreeModule, i1$1.PupaIconsModule, PupaTreeLayoutModule,
        PupaScrollableModule], exports: [TreeNewComponent, TreeItemTemplateDirective] });
PupaTreeNewModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeNewModule, imports: [CommonModule,
        PupaSkeletonModule,
        PupaSpinnerModule,
        ScrollingModule,
        CdkTreeModule,
        PupaIconsModule.forFeature([iosArrowDownIcon, iosArrowForwardIcon]),
        PupaTreeLayoutModule,
        PupaScrollableModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeNewModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [TreeNewComponent, TreeItemTemplateDirective],
                    imports: [
                        CommonModule,
                        PupaSkeletonModule,
                        PupaSpinnerModule,
                        ScrollingModule,
                        CdkTreeModule,
                        PupaIconsModule.forFeature([iosArrowDownIcon, iosArrowForwardIcon]),
                        PupaTreeLayoutModule,
                        PupaScrollableModule,
                    ],
                    providers: [],
                    exports: [TreeNewComponent, TreeItemTemplateDirective],
                }]
        }] });

const MODULES = [PupaTreeLayoutModule, PupaTreeNewModule];
class PupaTreeModule {
}
PupaTreeModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
PupaTreeModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeModule, imports: [PupaTreeLayoutModule, PupaTreeNewModule], exports: [PupaTreeLayoutModule, PupaTreeNewModule] });
PupaTreeModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeModule, imports: [MODULES, PupaTreeLayoutModule, PupaTreeNewModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PupaTreeModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: MODULES,
                    exports: MODULES,
                }]
        }] });

const DEFAULT_TREE_ITEM_SIZE_PX = 32;

class TreeDataDisplayCollection {
    constructor() {
        this.data$ = new BehaviorSubject([]);
        this.selectedIdsList$ = new BehaviorSubject([]);
        this.trackBy$ = new BehaviorSubject(TreeDataDisplayCollection.trackBy);
        this.scrollBehavior$ = new BehaviorSubject('smooth');
        this.expandedIdsList$ = new BehaviorSubject([]);
        this.hasDragAndDrop$ = new BehaviorSubject(false);
        this.treeItemSizePx$ = new BehaviorSubject(DEFAULT_TREE_ITEM_SIZE_PX);
        this.isLoading$ = new BehaviorSubject(false);
    }
    setExpandedIdsList(value) {
        this.expandedIdsList$.next(value);
    }
    setIsLoading(value) {
        this.isLoading$.next(value);
    }
    setData(data) {
        this.data$.next(data);
        return this.data$.pipe(take(1));
    }
    setSelectedIdsList(value) {
        this.selectedIdsList$.next(value);
    }
}
TreeDataDisplayCollection.trackBy = (index, dataItem) => {
    if (isNil(dataItem)) {
        return `${index}__null`;
    }
    return `${index}__${JSON.stringify(dataItem)}`;
};

class DefaultTreeEventHandler {
    constructor(eventBus, dataDisplayCollection) {
        this.eventBus = eventBus;
        this.dataDisplayCollection = dataDisplayCollection;
        this.subscription = new Subscription();
        this.reconnect();
    }
    subscribeToEvents() {
        this.subscription.add(this.getSubscriptionToSetData());
        this.subscription.add(this.getSubscriptionForUpdateItem());
        this.subscription.add(this.getSubscriptionForRemoveItem());
        this.subscription.add(this.getSubscriptionForScrollTo());
        this.subscription.add(this.getSubscriptionForRemoveChildren());
        this.subscription.add(this.getSubscriptionForSetChildren());
    }
    getSubscriptionToSetData() {
        return this.getEvents(TreeEvents.SetData)
            .pipe(switchMap((event) => this.dataDisplayCollection.setData(event.payload).pipe(mapTo(event))))
            .subscribe((event) => this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id)));
    }
    getSubscriptionForScrollTo() {
        return this.getEvents(TreeEvents.ScrollById)
            .pipe(withLatestFrom(this.dataDisplayCollection.data$))
            .subscribe(([event, data]) => {
            const treeItem = DefaultTreeEventHandler.getTreeItem(event.payload, data);
            if (isNil(treeItem)) {
                return this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
            }
            const index = data.indexOf(treeItem);
            this.eventBus.dispatch(new TreeEvents.ScrollByIndex(index));
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    getSubscriptionForRemoveItem() {
        return this.getEvents(TreeEvents.RemoveItem)
            .pipe(withLatestFrom(this.dataDisplayCollection.data$, this.dataDisplayCollection.expandedIdsList$))
            .subscribe(([event, data, expandedIdsList]) => {
            this.removeItemWithChildren(event.payload, data, expandedIdsList);
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    getSubscriptionForUpdateItem() {
        return this.getEvents(TreeEvents.UpdateItem)
            .pipe(withLatestFrom(this.dataDisplayCollection.data$))
            .subscribe(([event, data]) => {
            this.updateItem(event.payload, data);
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    updateItem(updatedItem, data) {
        const treeItemExists = DefaultTreeEventHandler.treeItemExists(updatedItem.id, data);
        if (!treeItemExists) {
            return;
        }
        const updatedData = data.map((treeItem) => {
            if (treeItem.id === updatedItem.id) {
                return new FlatTreeItem(treeItem.isExpandable, updatedItem.name, treeItem.level, treeItem.id, updatedItem.originalData, treeItem.isElement);
            }
            return treeItem;
        });
        this.eventBus.dispatch(new TreeEvents.SetData(updatedData));
    }
    removeItem(removeItemId, data) {
        const treeItemExists = DefaultTreeEventHandler.treeItemExists(removeItemId, data);
        if (!treeItemExists) {
            return;
        }
        const updatedData = data.filter((treeItem) => treeItem.id !== removeItemId);
        this.eventBus.dispatch(new TreeEvents.SetData(updatedData));
    }
    reconnect() {
        this.disconnect();
        this.subscribeToEvents();
    }
    disconnect() {
        this.subscription.unsubscribe();
        this.subscription = new Subscription();
    }
    getEvents(eventType) {
        return this.eventBus
            .listen()
            .pipe(filter((event) => event instanceof eventType));
    }
    getSubscriptionForSetChildren() {
        return this.getEvents(TreeEvents.SetChildren)
            .pipe(withLatestFrom(this.dataDisplayCollection.data$, this.dataDisplayCollection.expandedIdsList$))
            .subscribe(([event, data, expandedIdsList]) => {
            this.setChildren(event.payload.treeItemId, event.payload.children, data, expandedIdsList);
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    getSubscriptionForRemoveChildren() {
        return this.getEvents(TreeEvents.RemoveChildren)
            .pipe(withLatestFrom(this.dataDisplayCollection.data$, this.dataDisplayCollection.expandedIdsList$))
            .subscribe(([event, data, expandedIdsList]) => {
            this.removeChildren(event.payload, data, expandedIdsList);
            this.eventBus.dispatch(new QueueEvents.RemoveFromQueue(event.id));
        });
    }
    setChildren(parentId, children, data, expandedIdsList) {
        if (isNil(parentId)) {
            const newData = children.map((treeItem) => (Object.assign(Object.assign({}, treeItem), { level: 0 })));
            this.eventBus.dispatch(new TreeEvents.SetData(newData));
            this.eventBus.dispatch(new TreeEvents.SetExpanded([]));
            return;
        }
        const parentIsExpanded = expandedIdsList.includes(parentId);
        if (parentIsExpanded) {
            return;
        }
        const parent = DefaultTreeEventHandler.getTreeItem(parentId, data);
        const isExpanded = expandedIdsList.includes(parentId);
        if (isNil(parent) || isExpanded) {
            return;
        }
        const childrenIdsSet = new Set();
        const childrenWithLevel = children.map((childItem) => {
            childrenIdsSet.add(childItem.id);
            return Object.assign(Object.assign({}, childItem), { level: Number(parent.level) + 1 });
        });
        const parentIndex = data.indexOf(parent);
        const dataWithChildren = [
            ...data.slice(0, parentIndex),
            parent,
            ...childrenWithLevel,
            ...data.slice(parentIndex + 1),
        ];
        const noExpandedChildrenList = expandedIdsList.filter((expandedTreeItemId) => !childrenIdsSet.has(expandedTreeItemId));
        const newExpandedList = [...noExpandedChildrenList, parent.id];
        this.eventBus.dispatch(new TreeEvents.SetData(dataWithChildren));
        this.eventBus.dispatch(new TreeEvents.SetExpanded(newExpandedList));
    }
    removeItemWithChildren(removeItemId, data, expanded) {
        const treeItemExists = DefaultTreeEventHandler.treeItemExists(removeItemId, data);
        if (!treeItemExists) {
            this.eventBus.dispatch(new TreeEvents.SetData(data));
            this.eventBus.dispatch(new TreeEvents.SetExpanded(expanded));
            return;
        }
        const [dataWithoutChildren, expandedWithoutChildren] = this.getRemovedChildren(removeItemId, data, expanded);
        const dataWithoutRemovedItem = dataWithoutChildren.filter((treeItem) => treeItem.id !== removeItemId);
        this.eventBus.dispatch(new TreeEvents.SetData(dataWithoutRemovedItem));
        this.eventBus.dispatch(new TreeEvents.SetExpanded(expandedWithoutChildren));
    }
    removeChildren(parentId, data, expanded) {
        const [dataWithoutChildren, expandedWithoutChildren] = this.getRemovedChildren(parentId, data, expanded);
        this.eventBus.dispatch(new TreeEvents.SetData(dataWithoutChildren));
        this.eventBus.dispatch(new TreeEvents.SetExpanded(expandedWithoutChildren));
    }
    getRemovedChildren(parentId, data, expanded) {
        const parent = DefaultTreeEventHandler.getTreeItem(parentId, data);
        if (isNil(parent)) {
            return [data, expanded];
        }
        const parentIndex = data.indexOf(parent);
        const dataBeforeParent = parentIndex === 0 ? [] : data.slice(0, parentIndex);
        const dataAfterParent = data.slice(parentIndex + 1);
        const nextNonChildIndex = dataAfterParent.findIndex((dataItem) => dataItem.level <= parent.level);
        const isLastParent = nextNonChildIndex === -1;
        const children = dataAfterParent.slice(0, nextNonChildIndex);
        const childrenIdsList = children.map((child) => child.id);
        const expandedWithoutChildren = expanded.filter((expandedItemId) => !childrenIdsList.includes(expandedItemId) && expandedItemId !== parentId);
        const dataAfterLastChild = isLastParent ? [] : dataAfterParent.slice(nextNonChildIndex);
        const dataWithoutChildren = [...dataBeforeParent, parent, ...dataAfterLastChild];
        return [dataWithoutChildren, expandedWithoutChildren];
    }
    static treeItemExists(treeItemId, data) {
        const treeItem = DefaultTreeEventHandler.getTreeItem(treeItemId, data);
        return !isNil(treeItem);
    }
    static getTreeItem(treeItemId, data) {
        return data.find((treeItem) => treeItem.id === treeItemId);
    }
}

class TreeController {
    constructor(options) {
        this.options = options;
        this.eventBus = new EventBus();
        this.queue = new EventsQueue(this.eventBus);
        this.dataDisplayCollection = new TreeDataDisplayCollection();
        this.handler = new DefaultTreeEventHandler(this.eventBus, this.dataDisplayCollection);
        this.setScrollBehavior(options === null || options === void 0 ? void 0 : options.scrollBehavior);
        this.setTrackBy(options === null || options === void 0 ? void 0 : options.trackBy);
        this.setHasDragAndDrop(options === null || options === void 0 ? void 0 : options.hasDragAndDrop);
        this.setTreeItemSizePx(options === null || options === void 0 ? void 0 : options.treeItemSizePx);
    }
    dispatchInQueue(event) {
        const queueEvent = new QueueEvents.AddToQueue(event);
        this.eventBus.dispatch(queueEvent);
    }
    getOptions() {
        return this.options;
    }
    setData(data) {
        this.dispatchInQueue(new TreeEvents.SetData(data));
    }
    setSelected(...selectedIds) {
        this.dispatchInQueue(new TreeEvents.SetSelected(selectedIds));
    }
    setLoading(isLoading) {
        this.eventBus.dispatch(new TreeEvents.SetLoading(isLoading));
    }
    setHasDragAndDrop(hasDragAndDrop = false) {
        this.dataDisplayCollection.hasDragAndDrop$.next(hasDragAndDrop);
    }
    getEvents(eventType) {
        return this.queue.getEvents(eventType);
    }
    getDataDisplayCollectionRef() {
        return this.dataDisplayCollection;
    }
    expand(treeItemId) {
        this.eventBus.dispatch(new TreeEvents.Expand(treeItemId));
    }
    setChildren(treeItemId, children) {
        this.dispatchInQueue(new TreeEvents.SetChildren({ treeItemId, children }));
    }
    removeChildren(treeItemId) {
        this.dispatchInQueue(new TreeEvents.RemoveChildren(treeItemId));
    }
    removeTreeItem(treeItemId) {
        this.dispatchInQueue(new TreeEvents.RemoveItem(treeItemId));
    }
    setTreeItem(treeItem) {
        this.dispatchInQueue(new TreeEvents.UpdateItem(treeItem));
    }
    scrollTo(treeItemId) {
        this.dispatchInQueue(new TreeEvents.ScrollById(treeItemId));
    }
    setScrollBehavior(scrollBehavior = 'smooth') {
        this.dataDisplayCollection.scrollBehavior$.next(scrollBehavior);
    }
    setTrackBy(trackBy = TreeDataDisplayCollection.trackBy) {
        this.dataDisplayCollection.trackBy$.next(trackBy);
    }
    setTreeItemSizePx(treeItemSizePx = DEFAULT_TREE_ITEM_SIZE_PX) {
        this.dataDisplayCollection.treeItemSizePx$.next(treeItemSizePx);
    }
}

class TreeDragAndDropControl {
    constructor() {
        this.draggingHasStarted$ = new BehaviorSubject(false);
        this.droppedSubject$ = new Subject();
        this.draggableNode$ = new BehaviorSubject(null);
        this.scrollDirection$ = new BehaviorSubject(null);
        this.expandNodeWithDelay$ = new Subject();
        this.hasDragAndDrop$ = new BehaviorSubject(false);
        this.mouseDownPosition = null;
        this.draggableElementBoundingBox = null;
        this.draggableExpandableDescendants = [];
    }
    setHostAndRenderer(host, renderer) {
        this.host = host;
        this.renderer = renderer;
    }
    setHasDragAndDrop(hasDragAndDrop) {
        this.hasDragAndDrop$.next(hasDragAndDrop);
    }
    mouseDown(treeNode, event) {
        this.hasDragAndDrop$.pipe(take(1)).subscribe((hasDragAndDrop) => {
            if (!hasDragAndDrop) {
                return;
            }
            this.draggableNode$.next(treeNode);
            this.mouseDownPosition = {
                left: event.screenX,
                top: event.screenY,
            };
            if (event.target instanceof HTMLElement) {
                this.draggableElementBoundingBox = event.target.getBoundingClientRect();
            }
        });
    }
    mouseMove({ screenX, screenY }, draggableElement, data$) {
        if (isNil(this.mouseDownPosition)) {
            return;
        }
        this.draggingHasStarted$.pipe(take(1)).subscribe((draggingHasStarted) => {
            var _a;
            const dragDistanceSqr = Math.pow(this.mouseDownPosition.left - screenX, 2) + Math.pow(this.mouseDownPosition.top - screenY, 2);
            const druggingMustBeStarted = dragDistanceSqr > ((_a = this.draggableElementBoundingBox) === null || _a === void 0 ? void 0 : _a.height) / 2;
            if (!draggingHasStarted && druggingMustBeStarted) {
                this.startDragging(draggableElement, data$);
                this.draggingHasStarted$.next(true);
                this.updateDraggablePosition(screenX, screenY, draggableElement);
            }
            if (draggingHasStarted) {
                this.updateDraggablePosition(screenX, screenY, draggableElement);
            }
        });
    }
    mouseUp() {
        this.draggingHasStarted$
            .pipe(take(1), withLatestFrom(this.draggableNode$))
            .subscribe(([draggingHasStarted, draggableNode]) => {
            if (draggingHasStarted && !isNil(this.dropNode)) {
                this.droppedSubject$.next({
                    draggedElement: draggableNode,
                    droppedElement: this.dropNode,
                });
            }
            this.draggingHasStarted$.next(false);
            this.draggableNode$.next(null);
            this.dropNode = null;
            this.mouseDownPosition = null;
            this.draggableElementBoundingBox = null;
            this.scrollDirection$.next(null);
            this.draggableExpandableDescendants = [];
        });
    }
    mouseEnter(node) {
        this.draggingHasStarted$.pipe(take(1)).subscribe((draggingHasStarted) => {
            if (!draggingHasStarted) {
                return;
            }
            if (!node.isElement) {
                this.dropNode = node;
            }
            if (this.canDrop(node)) {
                this.expandNodeWithDelay$.next(node);
            }
        });
    }
    canDrop(node) {
        if (isNil(node)) {
            return false;
        }
        return node.isExpandable && !this.draggableExpandableDescendants.includes(node.id);
    }
    mouseLeave() {
        this.draggingHasStarted$.pipe(take(1)).subscribe((draggingHasStarted) => {
            if (!draggingHasStarted) {
                return;
            }
            this.dropNode = null;
            this.expandNodeWithDelay$.next(null);
        });
    }
    startDragging(draggableElement, data$) {
        this.setupWidthForDraggableElement(draggableElement);
        this.setupDraggableExpandableDescendants(data$);
    }
    setupWidthForDraggableElement(draggableElement) {
        timer(0)
            .pipe(take(1))
            .subscribe(() => {
            this.renderer.setStyle(draggableElement.nativeElement, 'width', `${this.draggableElementBoundingBox.width}px`);
        });
    }
    setupDraggableExpandableDescendants(data$) {
        data$
            .pipe(take(1), withLatestFrom(this.draggableNode$))
            .subscribe(([treeItems, draggableNode]) => {
            var _a;
            const targetLevel = (_a = draggableNode === null || draggableNode === void 0 ? void 0 : draggableNode.level) !== null && _a !== void 0 ? _a : 0;
            let currentIndex = treeItems.findIndex((dataPoint) => dataPoint.id === (draggableNode === null || draggableNode === void 0 ? void 0 : draggableNode.id)) + 1;
            const endOfListNotReached = () => treeItems.length !== currentIndex;
            const targetLevelNotReached = () => treeItems[currentIndex].level !== targetLevel;
            const result = [draggableNode === null || draggableNode === void 0 ? void 0 : draggableNode.id];
            while (endOfListNotReached() && targetLevelNotReached()) {
                const currentNode = treeItems[currentIndex];
                if (currentNode.isExpandable) {
                    result.push(currentNode.id);
                }
                currentIndex++;
            }
            this.draggableExpandableDescendants = result;
        });
    }
    updateDraggablePosition(screenX, screenY, draggableElement) {
        const draggableElementPositionShift = {
            left: this.mouseDownPosition.left - this.draggableElementBoundingBox.left,
            top: this.mouseDownPosition.top - this.draggableElementBoundingBox.top,
        };
        const bottomBorderPositionY = this.host.nativeElement.clientHeight - this.draggableElementBoundingBox.height;
        const draggableElementPosition = {
            left: screenX - this.draggableElementBoundingBox.left - draggableElementPositionShift.left,
            top: getClampedValue(screenY - draggableElementPositionShift.top, 0, bottomBorderPositionY),
        };
        const isTopBorderReached = draggableElementPosition.top <= this.draggableElementBoundingBox.height;
        const isBottomBorderReached = draggableElementPosition.top >= bottomBorderPositionY;
        this.scrollDirection$.next(isTopBorderReached ? 'up' : isBottomBorderReached ? 'down' : null);
        if (!isNil(draggableElement)) {
            this.renderer.setStyle(draggableElement.nativeElement, 'left', `${draggableElementPosition.left}px`);
            this.renderer.setStyle(draggableElement.nativeElement, 'top', `${draggableElementPosition.top}px`);
        }
    }
}
TreeDragAndDropControl.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeDragAndDropControl, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
TreeDragAndDropControl.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeDragAndDropControl });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: TreeDragAndDropControl, decorators: [{
            type: Injectable
        }] });

export { DEFAULT_TREE_ITEM_SIZE_PX, DefaultTreeEventHandler, FlatTreeItem, PupaTreeLayoutModule, PupaTreeModule, PupaTreeNewModule, TREE_NODE_OFFSET, TreeController, TreeDataDisplayCollection, TreeDragAndDropControl, TreeEvents, TreeItemTemplateDirective, TreeNewComponent, TreeNodeActionsDirective, TreeNodeComponent, TreeNodeSkeletonComponent, TreeRangedDataSource };
//# sourceMappingURL=bimeister-pupakit.tree.mjs.map
